/**
 * @file
 * Provides Outlayer Isotope loader.
 */

(function (Drupal, Isotope, _db) {

  'use strict';

  /**
   * Outlayer utility functions.
   *
   * @namespace
   */
  Drupal.outLayer.isotope = _db.extend({}, Drupal.outLayer || {}, {
    $empty: null,
    $items: [],
    $slicks: [],
    $filter: null,
    $sorter: null,
    $search: null,
    $input: null,
    isIsotope: false,
    useGridStack: true,
    dataFilter: null,
    searchString: null,
    activeFilters: [],
    activeSorters: [],
    gridHeight: 210 + 'px',
    onClass: 'isotope',

    /**
     * Initializes the Outlayer instance.
     *
     * Isotope is not inited during page load until one of the filter buttons
     * is hit. Unless when using Grid Custom (ungridstack) which disables
     * GridStack layout for non-responsive Grid Custom which simply means you
     * are on your own, need to layout the grid and make it responsive manually.
     *
     * @param {HTMLElement} el
     *   The .outlayer--isotope HTML element.
     */
    load: function (el) {
      var me = this;

      // Only initialize it if not already, or a force for ugridstack.
      if (me.$instance === null) {
        me.isIsotope = true;
        me.$instance = new Isotope(el, me.options);

        me.$instance.on('arrangeComplete', function (filteredItems) {
          // Add hints about empty results.
          if (me.$empty !== null) {
            var empty = filteredItems.length === 0;
            el.classList[empty ? 'add' : 'remove']('outlayer--empty');
            me.$empty.textContent = empty ? Drupal.t('No data found.') : '';
          }

          // Prevents annoying collapsed/ layout reflow during filtering.
          el.style.minHeight = me.gridHeight;

          if (!me.isEnabled(el)) {
            me.resetInline(el, true);
          }
        });
      }
    },

    onResizeEnd: function (e, el) {
      el.style.minHeight = '';
    },

    resetInline: function (el, reset) {
      var me = this;
      var gh = me.isNativeGrid ? '' : me.gridHeight;

      el = el || me.$el;
      reset = reset || true;

      me.rePosition(el, reset);
      el.style.height = gh;
      el.style.minHeight = reset ? '' : gh;
    },

    backToGridStack: function (el) {
      var me = this;

      me.$instance.destroy();

      // Reset to defaults.
      me.isIsotope = false;
      el.classList.remove('outlayer--empty');

      // We have no onDestroy event, and destroy is unfortunately unclean.
      // This causes unwanted transition, but at least not breaking layout.
      // Isotope is NOT immediately destroyed till transitions ends.
      var reset = function () {
        me.resetInline(el, true);
      };
      window.setTimeout(reset, 1100);
    },

    destroy: function (el) {
      var me = this;

      el = el || me.$el;

      // Only destroy it using gridstack to avoid breaking gridstack layouts.
      if (me.$instance !== null && typeof me.$instance !== 'undefined') {
        // Only destroy if using gridstack layout.
        // The layout is managed by GridStack, not Isotope.
        if (me.useGridStack) {
          me.backToGridStack(el);
        }
        else {
          // Reset filter in case one of its results empty.
          // The layout is managed by Isotope, not GridStack.
          me.$instance.arrange({filter: '', sortBy: ''});
        }
      }

      // Switch js-driven layouts to native CSS Grid if so required.
      el.classList.remove('is-ol-native');
      if (me.isNativeGrid) {
        el.classList.remove('is-gs-layout');
      }

      me.revalidate();

      // Slicks might be borked after being hidden during filtering, refresh.
      if (me.$slicks.length) {
        _db.forEach(me.$slicks, function (el) {
          el.slick.refresh();
        });
      }
    },

    /**
     * Revalidate items.
     */
    revalidate: function () {
      var me = this;
      var blazy = '.b-lazy:not(.b-loaded)';
      var $elms = me.$el.querySelector(blazy) === null ? [] : me.$el.querySelectorAll(blazy);

      // Revalidate blazy.
      if ($elms.length && me.isBlazy) {
        Drupal.blazy.init.load($elms);
      }
    }

  });

  /**
   * Outlayer utility functions.
   *
   * @param {HTMLElement} grid
   *   The Outlayer HTML element.
   */
  function doOutLayerIsotope(grid) {
    var me = Drupal.outLayer.isotope;
    var id = grid.getAttribute('data-instance-id');

    // Pass data to Drupal.outLayer.isotope for easy reference.
    me.$el = grid;
    me.options = _db.parse(grid.getAttribute('data-outlayer-isotope')) || {};
    me.style = window.getComputedStyle(grid);

    me.prepare(grid);

    var height = parseInt(me.style.getPropertyValue('height'));

    me.gridHeight = height + 'px';
    me.$filter = document.querySelector('.outlayer-list--filter[data-instance-id="' + id + '"]');
    me.$search = document.querySelector('.outlayer-list--search[data-instance-id="' + id + '"]');
    me.$sorter = document.querySelector('.outlayer-list--sorter[data-instance-id="' + id + '"]');
    me.$slicks = grid.querySelector('.slick__slider') === null ? [] : grid.querySelectorAll('.slick__slider');
    me.$empty = grid.querySelector('.outlayer__empty');

    /**
     * Filter elements on a button click.
     *
     * @param {Event} e
     *   The event triggering the filter.
     */
    function doFilter(e) {
      var btn = e.target;
      var $active = me.$filter.querySelector('.is-active');

      me.dataFilter = btn.getAttribute('data-filter');

      // Only initialize it once on an event to not mess up with gridstack.
      me.init();

      // Toggle the current active button class.
      toggleButton($active, btn);

      // Filter items.
      me.$instance.arrange({
        filter: me.dataFilter
      });
    }

    /**
     * Filter elements on a button click.
     *
     * @param {Event} e
     *   The event triggering the filter.
     */
    function doSearch(e) {
      var $target = e.target;
      var searchText = $target.value;

      me.$input = $target;
      if (me.$input.value === '') {
        doDestroy();

        return;
      }

      // Only initialize it once on an event to not destroy gridstack.
      me.init();

      me.searchString = Drupal.checkPlain(searchText.toLowerCase());

      me.$instance.arrange({
        filter: function () {
          var input = this;
          return input.textContent.toLowerCase().indexOf(me.searchString) !== -1 ? true : false;
        }
      });
    }

    /**
     * Sorter elements on a button click.
     *
     * @param {Event} e
     *   The event triggering the filter.
     */
    function doSorter(e) {
      var btn = e.target;
      var value = btn.getAttribute('data-sort-by');
      var $active = me.$sorter.querySelector('.is-active');

      // Only initialize it once on an event to not mess up with gridstack.
      me.init();

      // Toggle the current active button class.
      toggleButton($active, btn);

      me.$instance.arrange({sortBy: value});
    }

    /**
     * Toggle button states.
     *
     * @param {HTMLElement} $active
     *   The active button, if any.
     * @param {HTMLElement} $btn
     *   The current clicked button
     */
    function toggleButton($active, $btn) {
      // Toggle the current active button class.
      if ($active !== null) {
        $active.classList.remove('is-active');
      }

      $btn.classList.add('is-active');

      // Adjusts the class for absolute positioning required by Isotope.
      me.$el.classList.add('is-gs-layout');

      // If using native Grid, disable absolute positioning required by Isotope.
      // The .is-gs-layout basically means disabling native temporarily during
      // filtering to allow Isotope take over the layout to arrange items.
      // Once a .button--reset hit, let native Grid take over its layout again.
      // The best of both worlds: native Grid and dynamic filtering.
      // The first requires relative positioning, the latter absolute.
      if (me.isNativeGrid) {
        me.$el.classList.add('is-ol-native');

        if (_db.hasClass($btn, 'button--reset')) {
          me.$el.classList.remove('is-gs-layout');
        }
      }
    }

    /**
     * Destroy instance on a button click.
     */
    function doDestroy() {
      me.destroy();

      if (me.$input !== null) {
        me.$input.value = '';
      }
    }

    /**
     * Layout elements.
     */
    function outlay() {
      // Outlayer only triggered on events, not on init.
      if (me.$filter !== null) {
        _db.on(me.$filter, 'click', '.button--filter', doFilter);
      }

      if (me.$search !== null) {
        var onSearch = Drupal.debounce(doSearch, 250);
        _db.on(me.$search, 'keyup', '.form-text--search', onSearch);
      }

      if (me.$sorter !== null) {
        me.activeSorters = _db.parse(me.$sorter.getAttribute('data-sorters')) || {};
        _db.on(me.$sorter, 'click', '.button--sorter', doSorter);
      }

      _db.forEach([me.$filter, me.$search, me.$sorter], function (cn) {
        if (cn !== null) {
          _db.on(cn, 'click', '.button--reset', doDestroy);
        }
      });

      // Only initialize Isotope if not using gridstack grids.
      // If not using gridstack, or using native, layout items manually.
      if (me.isNativeGrid || me.isUnGridStack) {
        me.forceHeight = me.isNativeGrid;
      }

      // Prevents extra height calc if the sizer is not removed.
      if (me.isValid(me.$sizer)) {
        me.$sizer.classList.add('box--stamp');
      }

      // Only init if really ungridstack, not using GridStack layout. But
      // using the non-responsive Grid Custom option. So far Isotope is never
      // initialized till a filter, sorter or search button is hit.
      // If ungridstack, normal Isotope layout below is triggered.
      if (me.isUnGridStack) {
        me.useGridStack = false;

        me.init(grid);

        // Runs update(), including on resize event, as nobody helps here.
        me.buildOut(grid);
      }

      me.cleanUp(grid);
    }

    outlay();
  }

  /**
   * Attaches Outlayer behavior to HTML element.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.outLayerIsotope = {
    attach: function (context) {
      if ('length' in context) {
        context = context[0];
      }

      var elms = context.querySelectorAll('.outlayer--isotope:not(.is-isotope-on)');
      if (elms.length > 0) {
        _db.once(_db.forEach(elms, doOutLayerIsotope, context));
      }
    }
  };

}(Drupal, Isotope, dBlazy));
